@@ -19,6 +19,7 @@ void help(void);
 
 
 int signature_check = 1;
+int forced_charset = 0; /* Flag which disallow rtf parser override charset*/
 int wrap_margin = WRAP_MARGIN;
 int (*get_unicode_char)(FILE *f,long *offset,long fileend) =NULL;
 
@@ -59,6 +60,7 @@ int main(int argc, char **argv) {
 		switch (c) {
 			case 's':
 				check_charset(&source_csname,optarg);
+				forced_charset = 1;
 				break;
 			case 'd':
 				check_charset(&dest_csname,optarg);
@@ -86,7 +88,7 @@ int main(int argc, char **argv) {
 				break;
 			case 'm': {
 						  char *endptr;
-						  wrap_margin = strtol(optarg,&endptr,0);
+						  wrap_margin = (int)strtol(optarg,&endptr,0);
 						  if (*endptr) {
 							  fprintf(stderr,"Invalid wrap margin value `%s'\n",optarg);
 							  exit(1);
