@@ -14,7 +14,7 @@
 
 char *charset_path=CHARSETPATH;
 char *source_csname=SOURCE_CHARSET, *dest_csname=TARGET_CHARSET;
-short int * source_charset;
+uint16_t * source_charset;
 int unknown_as_hex=0;
 char bad_char[]=UNKNOWN_CHAR;
 CHARSET target_charset;
@@ -22,25 +22,8 @@ CHARSET target_charset;
 /* Converts char in input charset into unicode representation           */
 /* Should be converted to macro                                         */
 /************************************************************************/
-#define MBCS_MAX 1024
-struct mbcs_map {
-	int mbcs;
-	unsigned short unicode;
-};
-
-static struct mbcs_map *mbcs_charset = NULL;
-static int mbcs_count;
-int to_unicode (short int *charset, int c) {
-	if ((c > 0) && (c <= 255))
-		return charset[c];
-	else {
-		int i;
-		for (i = 0; i < mbcs_count; i++) {
-			if (mbcs_charset[i].mbcs == c)
-				return mbcs_charset[i].unicode;
-		}
-	}
-	return -1;
+int to_unicode (uint16_t *charset, int c) {
+	return charset[c];
 }
 /************************************************************************/
 /* Search inverse charset record for given unicode char and returns     */
@@ -63,13 +46,14 @@ CHARSET make_reverse_map(short int *charset) {
 	int i,j,k,l;
 	short int *p;   
 	if (! charset) {
+		free(newmap);
 		return NULL;
 	}	
 	for (i=0;i<256;i++) {
 		k= charset[i];
 		j=  (unsigned)k>>8;
 		if (!newmap[j]) {
-			newmap[j] = malloc(sizeof(short int *)*256);
+			newmap[j] = (short int *)malloc(sizeof(short int)*256);
 			if (!newmap[j]) {
 				fprintf(stderr,"Insufficient memory for  charset\n");
 				exit(1);
@@ -85,15 +69,11 @@ CHARSET make_reverse_map(short int *charset) {
 /************************************************************************/
 /* Reads charset file (as got from ftp.unicode.org) and returns array of*/
 /* 256 short ints (malloced) mapping from charset t unicode             */
-/* additional MBCS_MAX for mbcs mapping from charset to unicode         */
 /************************************************************************/
-short int * read_charset(const char *filename) {
+uint16_t * read_charset(const char *filename) {
 	char *path;
 	FILE *f;
-	int reset_mcbs = 1;
-	short int *new=calloc(sizeof(short int),256);
-	if (!mbcs_charset)
-		mbcs_charset=calloc(sizeof(struct mbcs_map),MBCS_MAX);
+	uint16_t *new;
 	int c;
 	long int uc;
 	path= find_file(stradd(filename,CHARSET_EXT),charset_path);
@@ -104,38 +84,25 @@ short int * read_charset(const char *filename) {
 	f=fopen(path,"rb");
 
 	if (!f) {
-		perror(path);
+		perror(path); 
 		return NULL;
 	}
 	if (input_buffer)
 		setvbuf(f,input_buffer,_IOFBF,FILE_BUFFER);
 	/* defaults */
+	new = calloc(sizeof(short int),256);
 	for (c=0;c<32;c++) {
 		new[c]=c;
 	}
 	while (!feof(f)) {
 		if (fscanf(f,"%i %li",&c,&uc)==2) {
-			// store first MBCS_MAX mbcs chars
-			if ((c >= 0x8000) && (c <= 0xFFFF)) {
-				if (reset_mcbs) {
-					reset_mcbs = 0;
-					mbcs_count = 0;
-				}
-				if (mbcs_count < MBCS_MAX) {
-					mbcs_charset[mbcs_count].mbcs = c;
-					mbcs_charset[mbcs_count].unicode = uc;
-					mbcs_count++;
-				}
-			} else
-//			if (c<0||c>255||uc<0||(uc>0xFEFE && uc!=0xFFFE)) {
-// shiftjis has values > 0xFEFE that are not FFFE
-			if (c<0||c>255||uc<0) {
+			if (c<0||c>255||uc<0||(uc>0xFEFE&& uc!=0xFFFE)) {
 				fprintf(stderr,"Invalid charset file %s\n",path);
 				fclose(f);
+				free(new);
 				return NULL;
 			}
-			else
-				new[c]=uc;
+			new[c]=uc;
 		}
 		while((fgetc(f)!='\n')&&!feof(f)) ;
 	}
@@ -199,7 +166,7 @@ int get_utf16msb (FILE *f,long *offset,long fileend) {
 
 int get_utf8 (FILE *f,long *offset,long fileend) {
 	unsigned char buf[3];
-	int d,c;
+	int c;
     int result;
 	result=catdoc_read(buf, 1, 1, f);
 	if (result<0) {
@@ -208,14 +175,13 @@ int get_utf8 (FILE *f,long *offset,long fileend) {
 	}	
 	if (result==0) return EOF;
 	c=buf[0];
-	d=0;
 	if (c<0x80) 
 		return c;
 	if (c <0xC0) 
 		return 0xfeff; /*skip corrupted sequebces*/
 	if (c <0xE0) {
 		if (catdoc_read(buf+1, 1, 1, f)<=0) return EOF;
-		return ((c & 0x1F)<<6 | ((char)buf[1] & 0x3F));
+		return (((c & 0x1F)<<6) | ((char)buf[1] & 0x3F));
 	}
 	if (c <0xF0) {
 		if (catdoc_read(buf+1, 1, 2, f)<=2) return (int)EOF;
@@ -286,7 +252,7 @@ char *to_utf8(unsigned int uc) {
 }    
 
 struct cp_map {
-	int codepage;
+	uint16_t codepage;
 	char *charset_name;
 };
 
@@ -328,12 +294,6 @@ const char *charset_from_codepage(unsigned int codepage) {
 		/* For UCS2 */
 		return "";
 	} else 
-	if (codepage == 932)
-		return "shiftjis";
-	else
-	if (codepage == 950)
-		return "big5";
-	else
 	if (codepage<10000) {
 		sprintf(buffer,"cp%d",codepage);
 		return buffer;
