@@ -29,14 +29,14 @@ extern int quote_mode;
 void print_sheet(void);
 void free_sheet(void);
 void print_value(unsigned char *value);
-char *format_double(char *rec,int offset,int format_code);
+char *format_double(unsigned char *rec,int offset,int format_code);
 char *format_int(int value,int format_code);
-char *format_rk(char *rec,short int format_code);
+char *format_rk(unsigned char *rec,short int format_code);
 char *gettypename(long rectype);
-void parse_sst(char *sstbuf,int bufsize);
-void process_item (int rectype, int reclen, char *rec); 
+void parse_sst(unsigned char *sstbuf,int bufsize);
+void process_item (int rectype, int reclen, unsigned char *rec); 
 unsigned char **allocate(int row,int col);
-char *copy_unicode_string(unsigned char **src);
+unsigned char *copy_unicode_string(unsigned char **src);
 char convert8to8(char *src,int count);
 char *convert16to8(char *src,int count);
 void do_table(FILE *input,char *filename);
